import math
from copy import copy
from polysurface import spline_double_divide, Offset, SurfaceFromCurves
from vector_util import (pmiddle, rotate_z, pdiff, rotate_x, rotate_y,
                        padd, pdist,
                        )

def make_points(offs, wd):
    if wd == 0:
        ow = offs
        offs = 0
    else:
        ow = offs*wd
    pdown = [ow, 0, -offs]
    pup = [ow, 0, offs]
    pfwd = [ow, -offs, 0]
    pback = [ow, offs, 0]
    pdb = padd(pdown, pback)
    pdf = padd(pdown, pfwd)
    puf = padd(pup, pfwd)
    pub = padd(pup, pback)
    for p in [pdb, pdf, puf, pub]:
        p[0] = ow
    pdb[1] = offs*0.8
    pdf[2] = -offs*0.4
    return pdown, pup, pfwd, pback, pdb, pdf, puf, pub

def pmid(p1, p2, p3, dist, inv=False):
    ang = 25
    p2 = copy(p2)
    p3 = copy(p3)
    p2[0] = p1[0]
    p3[0] = p1[0]
    pt = pmiddle(p1, p2, p3, dist)
    pt = rotate_z(pt, ang if inv else -ang)
    return pt

def calc_pipe_angle(p1, p2, p3, p4, sepdist):
    """ calculates an offset angle so that a pipe, when added, is at the
        mid-point to contact with another pipe.  this is all about the x-axis
    """
    dthin = 0.015 # TODO import from car_util
    p2 = copy(p2)
    p3 = copy(p3)
    p4 = copy(p4)
    p2[0] = p1[0]
    p3[0] = p1[0]
    p4[0] = p1[0]
    # calculate the vector diffs (p1 is centre)
    v1 = pdiff(p1, p2)
    v2 = pdiff(p1, p3)
    v2len = pdist(p1, p3) / 2
    print "v1-v2", v1, v2
    # work out the angle at which v1 is, and rotate v2 back by that amount
    v1ang = math.atan2(v1[1], v1[2])
    v1angd = math.degrees(v1ang)
    v2 = rotate_x(v2, -v1angd)
    print "v1ang", v1angd, v2, v1, rotate_x(v1, -v1angd)
    # now we want the distance between the two pipes
    #d = math.sin(v1ang) * v2[2] / 2
    d = v2[1] / 2
    c = d - dthin/2
    print "dist", d, c, v2len, d/v2len
    # now work out the angle of the scaled pipe.  this to get the pipe to
    # sit at a centre-line half-way between two parallel pipes.
    print "acos", math.asin(c / v2len)
    v2ang = math.degrees(math.asin(c / v2len))
    print "v2ang", v2ang, v2
    #exit(-1)
    # finally we have the angle of rotation at which a [0,0,sphererad] vector
    # can be rotated on the x-axis.  wheww.
    return v1angd + v2ang

def calc_pipe_offset(p1, p2, p3, p4, sepdist, inv=False, off1=False,
                     inv2=False):
    zang = 0 # XXX better to set to zero.
    angle = calc_pipe_angle(p1, p2, p3, p4, sepdist)
    jup = [0, 0, -1.0]
    if inv2:
        angle = -angle
        jup = [0, 0, -1.0]
    jup = rotate_z(jup, zang if inv else -zang)
    return rotate_x(jup, angle)

doorsillpts = []
doorlinepts = []

def calc_door_offsets(i, inv=False):
    rad = 0.023/2
    row1 = doorsillpts
    row2 = doorlinepts
    i1 = (i + 7) % 7
    i2 = (i + 1) % 7
    offs1 = calc_pipe_offset(row1[i2], row1[i1],
                             row2[i1], row2[i2],
                             rad, False, False, inv)
    offs2 = calc_pipe_offset(row1[i1], row1[i2],
                             row2[i2], row2[i1],
                             rad, True, False, inv)
    return offs1, offs2


